/*
  vpd_util - VPD support routines.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2002, 2003, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: util.c,v 1.18 2004/01/08 03:53:08 martins Exp $
*/
  
/* For pread, pwrite: */
#define _XOPEN_SOURCE 500

/* For asprintf: */
#define _GNU_SOURCE

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "util.h"

char * vpd_program_name = "ERROR";

void
die(const char * message)
{
	fprintf(stderr, "%s: %s\n", vpd_program_name, message);
	exit(1);
}

void
die2(const char * message, const char * sub_message)
{
	fprintf(stderr, "%s: %s (%s)\n",
		vpd_program_name, message, sub_message);
	exit(1);
}

char *
build_relative_filename(const char * dir,
			const char * relpath)
{
	char * ret;

	if (-1 == asprintf(&ret, "%s/%s", dir, relpath)) {
		ret = NULL;
	}
	return ret;
}

bool
file_read_contents(const  char   * pathname,
		          char  ** buf,
		          size_t * length,
		          int    * err)
{
	int fd;
	size_t n;
	size_t done = 0;

	const size_t chunk_size = 4096;
	int chunk_count = 1;

	*buf = NULL;

	if (NULL == pathname)
		fd = 0;
	else if (-1 == (fd = open(pathname, O_RDONLY)))
		goto bail;

	if (NULL == (*buf = malloc(chunk_size * chunk_count)))
		goto bail;
	
	/* - 1 everywhere because we're going to NUL-terminate. */
	while (0 < (n = read(fd, *buf + done,
			     (chunk_size * chunk_count - 1) - done))) {
		done += n;
		if (done == (chunk_size * chunk_count - 1)) {
			chunk_count++;
			if (NULL ==
			    (*buf = realloc(*buf, chunk_size * chunk_count)))
				goto bail;
		}
	}
	
	if (-1 == n)
		goto bail;

	(*buf)[done] = '\0';

	if (NULL != length) {
		*length = done;
	}

	close(fd);
	return true;

 bail:
	if (NULL != length) {
		*length = 0;
	}
	*err = errno;
	if (NULL != *buf) {
		free(*buf);
	}
	*buf = NULL;
	close(fd);
	return false;
}

void
file_relative_read_contents(const char   * dir,
			    const char   * relpath,
			          char  ** buf,
			          size_t * length)
{
	int err;
	char * file = build_relative_filename(dir, relpath);

	if (!file_read_contents(file, buf, length, &err)) {
		die2(strerror(err), file);
	}

	free(file);
}

bool
file_write_contents(const  char  * pathname,
		    const  char  * buf,
		           size_t  count,
		           int   * err)
{
	int fd;
	size_t n;
	size_t done = 0;

	if (-1 == (fd = open(pathname, O_CREAT|O_WRONLY|O_TRUNC, 0644)))
		goto bail;

	while ((done != count) &&
	       (-1 != (n = write(fd, buf + done, count - done)))) {
		done += n;
	}
	
	if (-1 == n)
		goto bail;

	close(fd);

	return true;

 bail:
	*err = errno;
	close(fd);
	return false;
}

void
file_relative_write_contents(const char * dir,
			     const char * relpath,
			     const char * buf,
			        size_t   count)
{
	int err;
	char * file = build_relative_filename(dir, relpath);

	if (!file_write_contents(file, buf, count, &err)) {
		die2(strerror(err), file);
	}

	free(file);
}

/* These should never give short reads or writes */
ssize_t
big_cool_pread(int fd, uint8_t *buf, size_t count, off_t offset)
{
	ssize_t total = 0;
	ssize_t last  = 0;

	do {
		last = pread(fd, buf + total, count - total, offset + total);
		if (-1 != last) {
			total += last;
		}
	} while ((0 < last) && (total < count));
	
	if ((-1 == last) && (0 == total)) {
		total = -1;
	}

	return total;
}

ssize_t
big_cool_pwrite(int fd, const uint8_t *buf, size_t count, off_t offset)
{
	ssize_t total = 0;
	ssize_t last  = 0;

	do {
		last = pwrite(fd, buf + total, count - total, offset + total);
		if (-1 != last) {
			total += last;
		}
	} while ((0 < last) && (total < count));
	
	if ((-1 == last) && (0 == total)) {
		total = -1;
	}

	return total;
}

const char *
string_array_lookup(const struct string_array * a,
		    const char * d,
		    int k)
{
	int i = 0;
	while (a[i].val != NULL) {
		if (a[i].key == k) {
			return a[i].val;
		}
		i++;
	}
	return d;
}
