/*
  pci_lookup - Convert PCI vendor and device ids to text.
  
  This file is part of the Linux lsvpd package.
  
  (C) Copyright IBM Corp. 2003, 2004
  
  Maintained by  Martin Schwenke <martins@au.ibm.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: pci_lookup.c,v 1.8 2004/01/08 03:57:00 martins Exp $
*/
  
/* For asprintf: */
#define _GNU_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include "util.h"

static void
usage()
{
	fprintf(stderr, "usage:\t%s pci.ids vendor [ device  [ \"subvendor subdevice\" ] ]\n", vpd_program_name);
	fprintf(stderr, "\t%s pci.ids \"C class\" [ subclass  [ prog-if ] ]\n", vpd_program_name);
	exit(1);
}

static char *
find_next_line(const char * s, const char * max)
{
	char * ret;

	ret = memchr(s, '\n', max - s);

	if (NULL != ret) {
		ret++;
	} else {
		ret = (char *) max;
	}

	return ret;
}

static char *
strip_and_return_to_newline(const char * s, const char * max)
{
	size_t len;
	char * loc = (char *) s;
	char * ret;

	while ((loc < max) && (*loc == ' ')) {
		loc++;
	}

	/* Copy and NUL-terminate. */
	s = find_next_line(loc, max);
	len = s - loc;
	if (-1 == asprintf(&ret, "%*.*s", (int) len, (int) len, loc)) {
		ret = NULL;
	} 

	return ret;
}

static char *
lookup_at_start_of_line(const char * data, const char * max,
			const char * prefix, const char * str,
			char ** out)
{
	char * ret = (char *) data;
	size_t len_prefix = strlen(prefix);
	size_t len_str = strlen(str);
	int yip = -99; /* Non-zero. */

	while (((ret + len_prefix + len_str) < max) &&
	       (('#' == *ret) || ('\n' == *ret) ||
		((0 == (strncmp(ret, prefix, len_prefix))) &&
		 (0 != (yip = strncmp(ret + len_prefix, str, len_str)))))) {
		ret = find_next_line(ret, max);
	}

	/* If we didn't find it then we've failed. */
	if (0 == yip) {
		*out = strip_and_return_to_newline(ret + len_prefix + len_str,
						   max);
		if (NULL == *out) {
			ret = NULL;
		}
	} else {
		ret = NULL;
	}
	
	return ret;
}

int
main(int argc, char ** argv)
{
	char * data = NULL;
	size_t len;
	int    err;
	char * s = NULL;
	char * max;
	char * out ;

	vpd_program_name = argv[0];

	if ((argc < 3) || (5 < argc)) {
		usage();
	}

	if (!file_read_contents(argv[1], &data, &len, &err)) {
		die2(strerror(err), argv[1]);
	}

	max = data + len;

	s = lookup_at_start_of_line(data, max, "", argv[2], &out);
	
	if ((NULL == s)) {
		out = NULL;
		goto bad;
	}

	if (3 == argc) {
		goto good;
	}

	free(out);
	s = find_next_line(s, max);
	s = lookup_at_start_of_line(s, max, "\t", argv[3], &out);

	if (NULL == s) {
		out = NULL;
		goto bad;
	}

	if (4 == argc) {
		goto good;
	}

	free(out);
	out = NULL;
	s = find_next_line(s, max);
	s = lookup_at_start_of_line(s, max, "\t\t", argv[4], &out);

	if (NULL == s) {
		out = NULL;
		goto bad;
	}

 good:
	fputs(out, stdout);
	free(out);
	free(data);
	return 0;

 bad:
	free(out);
	free(data);
	return 1;
}
