/*
  scsi_vpd_std - Extract standard SCSI VPD.
  
  (C) Copyright IBM Corp. 2002, 2003
  
  This file is part of the Linux lsvpd package.

  Maintained by  Martin Schwenke <martins@au.ibm.com>
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: scsi_vpd_std.c,v 1.15 2003/11/05 03:21:34 martins Exp $
*/

#define _GNU_SOURCE
  
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#include <sys/stat.h>
#include <sys/types.h>

#include "scsi_vpd_common.h"
#include "util.h"

#define ARRAY_LEN(x) (sizeof((x))/sizeof((x)[0]))

/* Very vaguely taken from linux-2.5.22/drivers/scsi/scsi.[ch]
 * There are only really 14 possible type, but filling it out to 16
 * removes the need for error checking. */
static const struct string_array scsi_device_types_short[] =
{
        {0x0, "disk"},
        {0x1, "tape"},
        {0x2, "printer"},
        {0x3, "processor"},
        {0x4, "worm"},
        {0x5, "cdrom"},
        {0x6, "scanner"},
        {0x7, "optical-disk"},
        {0x8, "jukebox"},
        {0x9, "communications"},
        {0xa, "graphics"},
        {0xb, "graphics"},
        {0xd, "enclosure"},
        {-1, NULL}
};


static const struct string_array scsi_device_types[] =
{
        {0x0, "Disk Drive"},
        {0x1, "Tape Drive"},
        {0x2, "Printer Device"},
        {0x3, "Processor Device"},
        {0x4, "Write-once Device"},
        {0x5, "CD-ROM Drive"},
        {0x6, "Scanner Device"},
        {0x7, "Optical Memory Device"},
        {0x8, "Medium Changer Device"},
        {0x9, "Communications Device"},
        {0xa, "Graphics Arts Pre-press Device"},
        {0xb, "Graphics Arts Pre-press Device"},
        {0xd, "Enclosure Services Device"},
        {-1, NULL}
};

#define SCSI_CDROM_DEFAULT "Other SCSI CD-ROM Drive"
static const struct string_array cdrom_device_types[] =
{
	{0, SCSI_CDROM_DEFAULT},
	{1, "OEM SCSI CD-ROM Drive"},
	{2, "SCSI CD-ROM Drive"},
	{3, "SCSI Multimedia CD-ROM Drive"},
	{4, "OEM SCSI DVD-ROM Drive"},
	{5, "SCSI DVD-ROM Drive"},
	{6, "SCSI DVD-RAM Drive"},
	{7, "OEM SCSI DVD-RAM Drive"},
	{-1, NULL}
};

static void
usage()
{
	fprintf(stderr, "usage: %s indir outdir\n", vpd_program_name);
	exit(1);
}


static char *
make_ds_value(const char * data, unsigned char dtype, const char * dir)
{
	char * data_0xc7 = NULL;
	size_t len_0xc7;
	int err;
	char  * file = NULL;

	char *s, *ret;

	/* Get info for naming quirks, such as LVD, Multimedia. */
	file = build_relative_filename(dir, "0xc7");
	file_read_contents(file, &data_0xc7, &len_0xc7, &err);
	free(file);

	/* Now do DS and type. */

	/* Check for SCSI *Multimedia* CD-ROM Drive. */
	if (dtype == 5) {
		int type = 0;
		if ((data_0xc7 != NULL) && (len_0xc7 >=34)) {
			type = (int) data_0xc7[33];
		}
		/* So it can be freed later. */
		asprintf(&s, "%s",
			 string_array_lookup(cdrom_device_types,	
					     SCSI_CDROM_DEFAULT,
					     type));
	} else {
		if ((dtype == 0) &&
		    (data_0xc7 != NULL) &&
		    (len_0xc7 >=35) && (data_0xc7[34] == 7)) {
			/* *LVD* SCSI Disk Drive. */
			asprintf(&s,
				 "LVD SCSI %s",
				 string_array_lookup (scsi_device_types,
						      "Unknown Device",
						      dtype));
		} else {
			asprintf(&s,
				 "SCSI %s",
				 string_array_lookup(scsi_device_types,
						     "Unknown Device",
						     dtype));
		}
	}

	asprintf(&ret, "%s%s",
		 ((data[7] & 0x40) ? "32 Bit " :
		  ((data[7] & 0x20) ? "16 Bit " : "")),
		 s);

	free(s);

	if (NULL != data_0xc7) {
		free(data_0xc7);
	}
	
	return ret;
}

int
main(int argc, char **argv)
{
	char * data = NULL;
	size_t len;

	unsigned char dtype;
	const char * dtype_short;
	char * dtype_str;

	vpd_program_name = argv[0];

	if (argc != 3) {
		usage();
	}

	/* Grab contents of inquiry file. */
	file_relative_read_contents(argv[1], "inquiry", &data, &len);

	/* Validate data. */
	if (len < 32) {
		fprintf(stderr, "%s: short standard inquiry data in %s/%s\n",
			vpd_program_name, argv[1], "inquiry");
		exit(1);
	}

	dtype = data[0] & 0x1F;
	dtype_short = string_array_lookup(scsi_device_types_short,
					  "unknown",
					  dtype);

	/* Dump standard fields. */
	dump_field(argv[2], "Z0", data,    8,  dtype_short);
	dump_field(argv[2], "MF", data+8,  8,  dtype_short);
	dump_field(argv[2], "TM", data+16, 16, dtype_short);

	/* Now do DS and type. */
	dtype_str = make_ds_value(data, dtype, argv[1]);
	dump_field(argv[2], "DS", dtype_str, strlen(dtype_str), dtype_short);
	free(dtype_str);

	free(data);

	file_relative_write_contents(argv[1], "type",
				     dtype_short, strlen(dtype_short));

	return 0;
}
